from __future__ import print_function, division, absolute_import

import numpy as np

import regreg.api as rr

from numpy import testing as npt
from numpy.testing import dec

@dec.setastest(True)
def test_logistic_counts():
    """
    Test the equivalence of binary/count specification in logistic_loglike
    """

    #Form the count version of the problem
    trials = np.random.binomial(5,0.5,100)+1
    successes = np.random.binomial(trials,0.5,len(trials)) 
    n = len(successes)
    p = 2*n
    X = np.random.normal(0,1,n*p).reshape((n,p))

    loss = rr.logistic_loglike.linear(X, successes=successes, trials=trials)
    penalty = rr.quadratic_loss(p, coef=1.)

    prob1 = rr.container(loss, penalty)
    solver1 = rr.FISTA(prob1)
    solver1.fit()
    solution1 = solver1.composite.coefs
    
    #Form the binary version of the problem
    Ynew = []
    Xnew = []

    for i, (s,n) in enumerate(zip(successes,trials)):
        Ynew.append([1]*s + [0]*(n-s))
        for j in range(n):
            Xnew.append(X[i,:])
    Ynew = np.hstack(Ynew)
    Xnew =  np.vstack(Xnew)

    loss = rr.logistic_loglike.linear(Xnew, successes=Ynew)
    penalty = rr.quadratic_loss(p, coef=1.)

    prob2 = rr.container(loss, penalty)
    solver2 = rr.FISTA(prob2)
    solver2.fit()
    solution2 = solver2.composite.coefs

   
    npt.assert_array_almost_equal(solution1, solution2, 3)


@dec.setastest(True)
def test_logistic_offset():
    """
    Test the equivalence of binary/count specification in logistic_likelihood
    """

    #Form the count version of the problem
    trials = np.random.binomial(5,0.5,10)+1
    successes = np.random.binomial(trials,0.5,len(trials)) 
    n = len(successes)
    p = 2*n

    X = np.hstack([np.ones((n,1)),np.random.normal(0,1,n*p).reshape((n,p))])

    loss = rr.logistic_loglike.linear(X, successes=successes, trials=trials)
    weights = np.ones(p+1)
    weights[0] = 0.
    penalty = rr.quadratic_loss.linear(weights, coef=.1, diag=True)

    prob1 = rr.container(loss, penalty)
    solver1 = rr.FISTA(prob1)
    vals = solver1.fit(tol=1e-12)
    solution1 = solver1.composite.coefs

    diff = 0.1

    loss = rr.logistic_loglike.affine(X, successes=successes, trials=trials, offset = diff*np.ones(n))
    weights = np.ones(p+1)
    weights[0] = 0.
    penalty = rr.quadratic_loss.linear(weights, coef=.1, diag=True)

    prob2 = rr.container(loss, penalty)
    solver2 = rr.FISTA(prob2)
    vals = solver2.fit(tol=1e-12)
    solution2 = solver2.composite.coefs

    ind = np.arange(1,p+1)

    print(solution1[np.arange(5)])
    print(solution2[np.arange(5)])

    npt.assert_array_almost_equal(solution1[ind], solution2[ind], 3)
    npt.assert_almost_equal(solution1[0]-diff,solution2[0], 2)






import numpy as np
from numpy import testing as npt
from numpy.testing import *
import regreg.api as rr

def test_multinomial_vs_logistic():

    """
    Test that multinomial regression with two categories is the same as logistic regression
    """

    n = 500
    p = 10
    J = 2

    X = np.random.standard_normal(n*p).reshape((n,p))
    counts = np.random.randint(0,10,n*J).reshape((n,J)) + 2

    mult_x = rr.linear_transform(X, input_shape=(p,J-1))
    loss = rr.multinomial_baseline_loglike.linear(mult_x, counts=counts)
    problem = rr.container(loss)
    solver = rr.FISTA(problem)
    solver.fit(debug=False, tol=1e-10)
    coefs1 = solver.composite.coefs

    loss = rr.logistic_loglike.linear(X, successes=counts[:,0], trials = np.sum(counts, axis=1))
    problem = rr.container(loss)
    solver = rr.FISTA(problem)
    solver.fit(debug=False, tol=1e-10)
    coefs2 = solver.composite.coefs

    loss = rr.logistic_loglike.linear(X, successes=counts[:,1], trials = np.sum(counts, axis=1))
    problem = rr.container(loss)
    solver = rr.FISTA(problem)
    solver.fit(debug=False, tol=1e-10)
    coefs3 = solver.composite.coefs

    npt.assert_equal(coefs1.shape,(p,J-1))
    npt.assert_array_almost_equal(coefs1.flatten(), coefs2.flatten(), 5)
    npt.assert_array_almost_equal(coefs1.flatten(), -coefs3.flatten(), 5)




from copy import copy

import nose.tools as nt
import numpy as np

from regreg.smooth import mglm, glm
from ...tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_gaussian_common():

    X = np.random.standard_normal((10,5))
    Y = np.random.standard_normal((10,3))

    for case_weights in [np.ones(10), None]:
        sat = mglm.stacked_common_loglike.gaussian(Y.T)
        L = mglm.mglm(X, sat, case_weights=case_weights)
        L.smooth_objective(np.zeros(L.shape), 'both')
        L_sub = L.subsample(np.arange(5))

        Xs = X[np.arange(5)]
        Ys = Y[np.arange(5)]

        beta = np.ones(L.shape)
        value_sub = 0.5 * np.linalg.norm(Ys - Xs.dot(beta))**2
        grad_sub = Xs.T.dot(Xs.dot(beta) - Ys)

        f, g = L_sub.smooth_objective(beta, 'both')

        np.testing.assert_allclose(value_sub, f)
        np.testing.assert_allclose(grad_sub, g)

        np.testing.assert_allclose(L.gradient(np.zeros(L.shape)),
                                   -X.T.dot(Y))

        Lcp = copy(L)

        L.objective(np.zeros(L.shape))
        L.latexify()

        L.saturated_loss.data

        L.data

        # check that subsample is getting correct answer

        Xsub = X[np.arange(5)]
        Ysub = Y[np.arange(5)]
        loss_sub = mglm.stacked_common_loglike.gaussian(Ysub.T)

        beta = np.ones(L.shape)
        if case_weights is not None:
            Lsub2 = mglm.mglm(Xsub, loss_sub, case_weights=case_weights[np.arange(5)])
            Lsub3 = mglm.mglm(Xsub, loss_sub, case_weights=case_weights[np.arange(5)])
        else:
            Lsub2 = mglm.mglm(Xsub, loss_sub)
            Lsub3 = mglm.mglm(Xsub, loss_sub)

        Lsub3.coef *= 2.

        f2, g2 = Lsub2.smooth_objective(beta, 'both')
        f3, g3 = Lsub3.smooth_objective(beta, 'both')

        np.testing.assert_allclose(f3, 2*f2)
        np.testing.assert_allclose(g3, 2*g2)

        beta = np.ones(L.shape)

        np.testing.assert_allclose(L_sub.gradient(beta),
                                   Lsub2.gradient(beta))

@set_seed_for_test()
def test_multinomial():

    """
    Test that multinomial regression with two categories is the same as logistic regression
    """

    n = 500
    p = 10
    J = 4


    X = np.random.standard_normal(n*p).reshape((n,p))
    counts = np.random.randint(0,10,n*J).reshape((n,J)) + 2
    for case_weights in [np.ones(n), None]:

        sat = mglm.multinomial_loglike(counts.shape, counts)
        L = mglm.mglm(X, 
                      sat,
                      case_weights=case_weights)
        L.smooth_objective(np.zeros(L.shape), 'both')
        L_sub = L.subsample(np.arange(100))

        np.testing.assert_allclose(L.gradient(np.zeros(L.shape)),
                                   -X.T.dot(counts - counts.mean(1)[:,None]))

        Lcp = copy(L)

        L.objective(np.zeros(L.shape))
        L.latexify()

        L.saturated_loss.data

        L.data

        # check that subsample is getting correct answer

        Xsub = X[np.arange(100)]
        counts_sub = counts[np.arange(100)]
        loss_sub = mglm.multinomial_loglike(counts_sub.shape, counts_sub)

        beta = np.ones(L.shape)
        if case_weights is not None:
            Lsub2 = mglm.mglm(Xsub, loss_sub, case_weights=case_weights[np.arange(100)])
            Lsub3 = mglm.mglm(Xsub, loss_sub, case_weights=case_weights[np.arange(100)])
        else:
            Lsub2 = mglm.mglm(Xsub, loss_sub)
            Lsub3 = mglm.mglm(Xsub, loss_sub)

        Lsub3.coef *= 2.

        f2, g2 = Lsub2.smooth_objective(beta, 'both')
        f3, g3 = Lsub3.smooth_objective(beta, 'both')

        np.testing.assert_allclose(f3, 2*f2)
        np.testing.assert_allclose(g3, 2*g2)

        beta = np.ones(L.shape)

        np.testing.assert_allclose(L_sub.gradient(beta),
                                   Lsub2.gradient(beta))

@set_seed_for_test()
def test_multinomial_baseline():

    """
    Test that multinomial regression with two categories is the same as logistic regression
    """

    n = 500
    p = 10
    J = 4


    X = np.random.standard_normal(n*p).reshape((n,p))
    counts = np.random.randint(0,10,n*J).reshape((n,J)) + 2
    for case_weights in [np.ones(n), None]:

        sat = mglm.multinomial_baseline_loglike((n, J-1), counts)
        L = mglm.mglm(X, 
                      sat,
                      case_weights=case_weights)
        L.smooth_objective(np.zeros(L.shape), 'both')
        L_sub = L.subsample(np.arange(100))

        np.testing.assert_allclose(L.gradient(np.zeros(L.shape)),
                                   -X.T.dot(counts - counts.mean(1)[:,None])[:,:(J-1)])

        Lcp = copy(L)

        L.objective(np.zeros(L.shape))
        L.latexify()

        L.saturated_loss.data

        L.data

        # check that subsample is getting correct answer

        Xsub = X[np.arange(100)]
        counts_sub = counts[np.arange(100)]
        loss_sub = mglm.multinomial_baseline_loglike((counts_sub.shape[0], J-1), counts_sub)

        beta = np.ones(L.shape)
        if case_weights is not None:
            Lsub2 = mglm.mglm(Xsub, loss_sub, case_weights=case_weights[np.arange(100)])
            Lsub3 = mglm.mglm(Xsub, loss_sub, case_weights=case_weights[np.arange(100)])
        else:
            Lsub2 = mglm.mglm(Xsub, loss_sub)
            Lsub3 = mglm.mglm(Xsub, loss_sub)

        Lsub3.coef *= 2.

        f2, g2 = Lsub2.smooth_objective(beta, 'both')
        f3, g3 = Lsub3.smooth_objective(beta, 'both')

        np.testing.assert_allclose(f3, 2*f2)
        np.testing.assert_allclose(g3, 2*g2)

        beta = np.ones(L.shape)

        np.testing.assert_allclose(L_sub.gradient(beta),
                                   Lsub2.gradient(beta))






import itertools

import numpy as np, regreg.api as rr
from .. import mixed_lasso as ml
from .. import group_lasso as gl
from ...tests.decorators import set_seed_for_test
from .test_seminorms import Solver, all_close, SolverFactory


@set_seed_for_test()
def test_mixed_lasso():

    prox_center = np.array([1,3,5,7,-9,3,4,6,7,8,9,11,13,4,-23,40], np.float)
    l1_penalty = np.array([0,1], np.intp)
    unpenalized = np.array([2,3], np.intp)
    positive_part = np.array([4,5], np.intp)
    nonnegative = np.array([], np.intp)
    mixed_spec = np.array([ml.L1_PENALTY,
                           ml.L1_PENALTY,
                           ml.UNPENALIZED,
                           ml.UNPENALIZED,
                           ml.POSITIVE_PART,
                           ml.POSITIVE_PART] + [0]*5 + [1]*5).astype(np.intp)
    weights = np.array([1.,1.])

    lagrange = 1.
    lipschitz = 0.5

    result = np.zeros_like(prox_center)

    result[unpenalized] = prox_center[unpenalized]
    result[positive_part] = (prox_center[positive_part] - lagrange / lipschitz) * np.maximum(prox_center[positive_part] - lagrange / lipschitz, 0)
    result[l1_penalty] = np.maximum(np.fabs(prox_center[l1_penalty]) - lagrange / lipschitz, 0) * np.sign(prox_center[l1_penalty])

    result[6:11] = prox_center[6:11] / np.linalg.norm(prox_center[6:11]) * max(np.linalg.norm(prox_center[6:11]) - weights[0] * lagrange/lipschitz, 0)
    result[11:] = prox_center[11:] / np.linalg.norm(prox_center[11:]) * max(np.linalg.norm(prox_center[11:]) - weights[1] * lagrange/lipschitz, 0)

    norms = np.zeros_like(weights)
    factors = np.zeros_like(weights)
    projection = np.zeros(prox_center.shape)

    prox_result = ml.mixed_lasso_lagrange_prox(prox_center, 
                                               lagrange, 
                                               lipschitz, 
                                               l1_penalty, 
                                               unpenalized, 
                                               positive_part, 
                                               nonnegative, 
                                               mixed_spec, 
                                               weights,
                                               norms,
                                               factors,
                                               projection)

    np.testing.assert_allclose(result, prox_result)

    norms = np.zeros_like(weights)
    factors = np.zeros_like(weights)
    projection = np.zeros(prox_center.shape)

    a3 = ml.mixed_lasso_lagrange_prox(prox_center, 
                                      lagrange, 
                                      1,
                                      l1_penalty, 
                                      unpenalized, 
                                      positive_part, 
                                      nonnegative, 
                                      mixed_spec, 
                                      weights,
                                      norms,
                                      factors,
                                      projection)

    norms = np.zeros_like(weights)
    factors = np.zeros_like(weights)
    projection = np.zeros(prox_center.shape)

    b3 = ml.mixed_lasso_dual_bound_prox(prox_center,
                                        lagrange,
                                        l1_penalty, 
                                        unpenalized, 
                                        positive_part, 
                                        nonnegative, 
                                        mixed_spec, 
                                        weights,
                                        norms,
                                        factors,
                                        projection)

    np.testing.assert_allclose(prox_center, a3 + b3)

    norms = np.zeros_like(weights)
    value = ml.seminorm_mixed_lasso(prox_center,
                                    l1_penalty, 
                                    unpenalized, 
                                    positive_part, 
                                    nonnegative, 
                                    mixed_spec, 
                                    weights,
                                    norms,
                                    0)

    test_value = 0
    test_value += np.fabs(prox_center[l1_penalty]).sum()
    test_value += np.fabs(np.maximum(prox_center[positive_part], 0)).sum()
    test_value += np.linalg.norm(prox_center[6:11])
    test_value += np.linalg.norm(prox_center[11:16])
    np.testing.assert_allclose(value, test_value)

    dual_value = ml.seminorm_mixed_lasso_dual(prox_center,
                                              l1_penalty, 
                                              unpenalized, 
                                              positive_part, 
                                              nonnegative, 
                                              mixed_spec, 
                                              weights,
                                              norms,
                                              0)


    norms = np.zeros_like(weights)
    factors = np.zeros_like(weights)
    projection = np.zeros(prox_center.shape)

    ml.mixed_lasso_bound_prox(prox_center,
                              0.5,
                              l1_penalty, 
                              unpenalized, 
                              positive_part, 
                              nonnegative, 
                              mixed_spec, 
                              weights,
                              norms,
                              factors,
                              projection)

    epigraph_center = np.zeros(prox_center.shape[0] + 1)
    epigraph_center[:-1] = prox_center
    epigraph_center[-1] = 0.5 * test_value

    norms = np.zeros_like(weights)
    factors = np.zeros_like(weights)
    projection = np.zeros(prox_center.shape)

    ml.mixed_lasso_epigraph(epigraph_center,
                            l1_penalty, 
                            unpenalized, 
                            positive_part, 
                            nonnegative, 
                            mixed_spec, 
                            weights,
                            norms,
                            factors,
                            projection)

@set_seed_for_test()
def test_compare_to_group_lasso():

    ps = np.array([0]*5 + [3]*3)
    weights = {3:2., 0:2.3}

    lagrange = 1.5
    lipschitz = 0.2
    p = gl.group_lasso(ps, weights=weights, lagrange=lagrange)
    z = 30 * np.random.standard_normal(8)
    q = rr.identity_quadratic(lipschitz, z, 0, 0)

    x = p.solve(q)

    norms = np.zeros(2)
    factors = np.zeros(2)
    projection = np.zeros(z.shape)

    a = ml.mixed_lasso_lagrange_prox(z, 
                                     lagrange, 
                                     lipschitz, 
                                     np.array([], np.intp), 
                                     np.array([], np.intp), 
                                     np.array([], np.intp), 
                                     np.array([], np.intp), 
                                     np.array([0,0,0,0,0,1,1,1]).astype(np.intp), 
                                     np.array([np.sqrt(5), 2]),
                                     norms,
                                     factors,
                                     projection)

    result = np.zeros_like(a)
    result[:5] = z[:5] / np.linalg.norm(z[:5]) * max(np.linalg.norm(z[:5]) - weights[0] * lagrange/lipschitz, 0)
    result[5:] = z[5:] / np.linalg.norm(z[5:]) * max(np.linalg.norm(z[5:]) - weights[3] * lagrange/lipschitz, 0)

    lipschitz = 1.
    q = rr.identity_quadratic(lipschitz, z, 0, 0)
    x2 = p.solve(q)
    pc = p.conjugate
    a2 = pc.solve(q)

    np.testing.assert_allclose(z-a2, x2)

    # make sure dtypes are correct

    norms = np.zeros(2)
    factors = np.zeros(2)
    projection = np.zeros(z.shape)

    a3 = ml.mixed_lasso_lagrange_prox(z, 
                                      lagrange, 
                                      1,
                                      np.array([], np.intp), 
                                      np.array([], np.intp), 
                                      np.array([], np.intp), 
                                      np.array([], np.intp), 
                                      np.array([0,0,0,0,0,1,1,1], np.intp), 
                                      np.array([np.sqrt(5), 2]),
                                      norms,
                                      factors,
                                      projection)

    norms = np.zeros(2)
    factors = np.zeros(2)
    projection = np.zeros(z.shape)

    b3 = ml.mixed_lasso_dual_bound_prox(z, 
                                        lagrange, 
                                        np.array([], np.intp), 
                                        np.array([], np.intp), 
                                        np.array([], np.intp), 
                                        np.array([], np.intp), 
                                        np.array([0,0,0,0,0,1,1,1], np.intp), 
                                        np.array([np.sqrt(5), 2]),
                                        norms,
                                        factors,
                                        projection)
    
    np.testing.assert_allclose(z, a3 + b3)


class MixedSolverFactory(SolverFactory):

    mixed_spec = [np.array([ml.L1_PENALTY,
                            ml.L1_PENALTY,
                            ml.UNPENALIZED,
                            ml.UNPENALIZED,
                            ml.POSITIVE_PART,
                            ml.POSITIVE_PART] + [0]*5 + [1]*5).astype(np.intp)]
    weights = [{0:1.,1:1.},{0:0.,1:1.}]
    lagrange = [1.1]
    FISTA_choices = [True]
    L_choices = [0.3]
    coef_stop_choices = [False]

    def __init__(self, klass, mode):
        self.klass = klass
        self.mode = mode

    def __iter__(self):
        pen_choices = itertools.product(self.weights,
                                        self.lagrange,
                                        self.mixed_spec)
        for offset, FISTA, coef_stop, L, q, pen in itertools.product(self.offset_choices,
                                                                     self.FISTA_choices,
                                                                     self.coef_stop_choices,
                                                                     self.L_choices,
                                                                     self.quadratic_choices,
                                                                     pen_choices):
            self.FISTA = FISTA
            self.coef_stop = coef_stop
            self.L = L

            w, l, spec = pen
            atom = self.klass(spec, l, weights=w)

            if q: 
                atom.quadratic = rr.identity_quadratic(0, 0, np.random.standard_normal(atom.shape)*0.02)

            if offset:
                atom.offset = 0.02 * np.random.standard_normal(atom.shape)

            solver = Solver(atom, interactive=self.interactive, 
                            coef_stop=coef_stop,
                            FISTA=FISTA,
                            L=L)
            yield solver

@set_seed_for_test()
@np.testing.dec.slow
def test_proximal_maps():
    for klass, mode in zip([ml.mixed_lasso, 
                            ml.mixed_lasso_dual], 
                           ['lagrange', 'bound']):
        factory = MixedSolverFactory(klass, mode)
        for solver in factory:
            for t in solver.all_tests():
                yield t
        factory = MixedSolverFactory(klass, mode)
        for solver in factory:
            for t in solver.all_tests():
                yield t



import numpy as np

have_sklearn = True
try:
    from .base import (sklearn_regression,
                       sklearn_regression_lagrange,
                       sklearn_classifier,
                       sklearn_classifier_lagrange)
except ImportError:
    have_sklearn = False

from ..smooth.mglm import (mglm, 
                           multinomial_loglike)

if have_sklearn:
    class sklearn_multinomial(sklearn_regression):

        """

        A simple regression mixin for sklearn
        that allows any atom to be used as a regularizer.

        """

        def _loglike_factory(self, X, y):
            response, case_weights_, offset_ = self._check_y_arg(y)

            successes = response

            return mglm.multinomial(X, 
                                    successes,
                                    case_weights=case_weights_,
                                    coef=self.coef,
                                    saturated_offset=offset_)

        def _saturated_score(self,
                             predictions,
                             response,
                             case_weights=None):

            successes = response

            loss = lambda yhat: multinomial_loglike(successes.shape,
                                                    successes,
                                                    case_weights=case_weights).smooth_objective(yhat, 'func')

            # factor of 2 to form proper deviance (default is negative log-likelihood,
            # while deviance is 2 * negative log-likelihood
            # negative sign is to align with sklearn's maximizing a score with grid search

            if self.score_method == 'deviance':
                return -2 * loss(predictions)
            elif self.score_method == 'mean_deviance':
                return -2 * loss(predictions) / predictions.shape[0]
            elif self.score_method == 'R2':
                SSE = 2 * loss(predictions)
                pi_0 = np.mean(response, 0)
                multinom_0 = np.log(pi_0)
                SST = 2 * loss(np.multiply.outer(multinom_0,  np.ones_like(response)))
                return 1 - SSE / SST
            elif self.score_method == 'accuracy':
                labels = predictions > 0
                return np.mean(labels == response)
            else:
                return np.nan

    class sklearn_multinomial_lagrange(sklearn_regression_lagrange, sklearn_multinomial):
        pass

    class sklearn_multinomial_classifier(sklearn_classifier):

        """

        A simple regression mixin for sklearn
        that allows any atom to be used as a regularizer.

        """

        def _loglike_factory(self, X, y):
            response, case_weights_, offset_ = self._check_y_arg(y)

            successes = response

            return mglm.multinomial(X, 
                                    successes,
                                    case_weights=case_weights_,
                                    coef=self.coef,
                                    saturated_offset=offset_ # should be None
                                    )

        def _saturated_score(self,
                             predictions,
                             response,
                             case_weights=None):

            successes = response

            if self.score_method == 'accuracy':
                return np.mean(predictions == successes)
            return np.nan

        def predict(self, X):
            """
            Predict labels in classification setting.

            Parameters
            ----------

            X : np.ndarray((n, p))
                Feature matrix.

            Returns
            -------

            probs : np.ndarray(n)
                Predictions from classification model.

            """
            linpred = X.dot(self._coefs)
            return np.argmax(linpred, 1)

        def predict_proba(self, X):
            """
            Predict new probabilities in classification setting.

            Parameters
            ----------

            X : np.ndarray((n, p))
                Feature matrix.

            Returns
            -------

            probs : np.ndarray(n)
                Predictions from classification model.

            """
            linpred = X.dot(self._coefs)
            exp_lin = np.exp(linpred)
            return exp_lin / exp_lin.sum(1)

    class sklearn_multinomial_classifier_lagrange(sklearn_multinomial_classifier):
        pass




import numpy as np

from joblib.parallel import Parallel, delayed

have_sklearn = True
try:
    from sklearn.model_selection import check_cv
except ImportError:
    have_sklearn = False
    pass

from . import strong_rules

def cross_validate(path_obj,
                   lagrange_seq,
                   path_fitter=strong_rules,
                   initial_data=None,
                   cv=5,
                   group_labels=None,
                   n_jobs=-1,
                   score=lambda x,y: np.linalg.norm(x-y)**2 / y.shape[0],
                   verbose=True,
                   inner_tol=1.e-5):

    '''
    Cross validates a path fitter over a set of 
    Lagrange parameters.

    Adjusts coefficient of subsampled loss by fraction ntrain / nsample.

    Parameters
    ----------

    path_obj : group_path
    '''

    # assumes that full data of path_obj.saturated_loss
    # is the same as

    # Compute paths for all folds 

    response = path_obj.saturated_loss.data
    nsample = response.shape[0]
    have_sklearn = False
    if have_sklearn:
        cv = check_cv(cv)
        folds = list(cv.split(range(nsample), range(nsample)))
    else:
        if type(cv) != type(3):
            raise ValueError('without sklearn, cv should be an integer')
        groups = np.floor(np.arange(nsample)  / nsample * cv).astype(np.int)
        np.random.shuffle(groups)
        folds = [(groups != g, groups == g)
                 for g in np.unique(groups)]

    # Loop over folds, computing path
    # for each (train, test)

    if initial_data is None:
        initial_data = (path_obj.solution,
                        path_obj.grad_solution)

    def fit_fold(path_obj,
                 lagrange_seq,
                 initial_data,
                 test,
                 train,
                 inner_tol):
        train_path = path_obj.subsample(train)
        train_path.saturated_loss.coef *= train.shape[0] / train.sum()
        train_results = path_fitter(train_path, 
                                    lagrange_seq, 
                                    initial_data,
                                    inner_tol=inner_tol)
        return train_results

    jobs = (delayed(fit_fold)(path_obj,
                              lagrange_seq,
                              initial_data,
                              test,
                              train,
                              inner_tol)
            for train, test in folds)

    # Execute the jobs in parallel using joblib
    results = Parallel(n_jobs=n_jobs,
                       verbose=verbose,
                       prefer="threads")(jobs)

    # Form the linear predictors and responses

    linpred_responses = [(path_obj.linpred(result['beta'], 
                                              path_obj.X,
                                              train_test[1]),
                          response[train_test[1]])
                         for train_test, result in zip(folds, results)]
    scores = [[score(linpred.T[i].T, y) for i in range(linpred.shape[-1])] for (linpred, y) in linpred_responses]

    return np.mean(scores, 0), np.std(scores, 0)


def cross_validate_alt(path_obj,
                       fit_fold,
                       fit_args,
                       lagrange_seq,
                       path_fitter=strong_rules,
                       initial_data=None,
                       cv=5,
                       group_labels=None,
                       n_jobs=-1,
                       score=lambda x,y: np.linalg.norm(x-y)**2 / y.shape[0],
                       verbose=True,
                       inner_tol=1.e-5):

    '''
    Cross validates a path fitter over a set of 
    Lagrange parameters.

    Adjusts coefficient of subsampled loss by fraction ntrain / nsample.

    Parameters
    ----------

    path_obj : group_path
    '''

    # assumes that full data of path_obj.saturated_loss
    # is the same as

    # Compute paths for all folds 

    response = path_obj.saturated_loss.data
    nsample = response.shape[0]
    have_sklearn = False
    if have_sklearn:
        cv = check_cv(cv)
        folds = list(cv.split(range(nsample), range(nsample)))
    else:
        if type(cv) != type(3):
            raise ValueError('without sklearn, cv should be an integer')
        groups = np.floor(np.arange(nsample)  / nsample * cv).astype(np.int)
        np.random.shuffle(groups)
        folds = [(groups != g, groups == g)
                 for g in np.unique(groups)]

    if initial_data is None:
        initial_data = (path_obj.solution,
                        path_obj.grad_solution)

    # Loop over folds, computing path
    # for each (train, test)

    jobs = (delayed(fit_fold)(lagrange_seq,
                              test,
                              train,
                              path_fitter,
                              initial_data,
                              *fit_args)
            for train, test in folds)

    # Execute the jobs in parallel using joblib
    results = Parallel(n_jobs=n_jobs,
                       verbose=verbose,
                       prefer="threads")(jobs)

    # Form the linear predictors and responses

    linpred_responses = [(path_obj.linpred(result['beta'], 
                                           path_obj.X,
                                           train_test[1]),
                          response[train_test[1]])
                         for train_test, result in zip(folds, results)]
    scores = [[score(linpred.T[i].T, y) for i in range(linpred.shape[-1])] for (linpred, y) in linpred_responses]

    return np.mean(scores, 0), np.std(scores, 0)






""" Test doc templater
"""

from regreg.doctemplates import (
    doc_templater,
    doc_template_provider,
    doc_template_user)

from nose.tools import assert_equal, assert_raises

def test_basic():
    @doc_templater()
    class C(object):
        @doc_template_provider
        def f(self):
            "A docstring %(var)s"

    @doc_templater(dict(var='something'))
    class D(C):
        @doc_template_user
        def f(self):
            pass

    assert_equal(D().f.__doc__, 'A docstring something')


def test_more_fully():
    @doc_templater()
    class C(object):
        @doc_template_provider
        def fa(self, arg):
            """A docstring %(a)s"""

        @doc_template_provider
        def fb(self, arg):
            """B docstring %(b)s"""

    # Providing a template sets the docstring to None
    assert_equal(C().fa.__doc__, None)

    @doc_templater(dict(a = 2, b='two'))
    class D(C):
        @doc_template_user
        def fa(self, arg):
            pass

        @doc_template_user
        def fb(self, arg):
            pass

    # Using a template
    assert_equal(D().fa.__doc__, "A docstring 2")
    assert_equal(D().fb.__doc__, "B docstring two")

    @doc_templater(dict(a = 3))
    class E(D):
        @doc_template_user
        def fa(self, arg):
            pass

        @doc_template_user
        def fb(self, arg):
            pass

    # Overriding a parameter in the template
    assert_equal(E().fa.__doc__, "A docstring 3")
    assert_equal(E().fb.__doc__, "B docstring two")

    def missing_param_error():
        @doc_templater(dict(a = 4))
        class Q(object):
            @doc_template_user
            def fb(self, arg):
                pass

    # Parameters inherited from subclass - missing - keyerror
    assert_raises(KeyError, missing_param_error)

    @doc_templater(dict(a = 4))
    class E(D):
        @doc_template_user
        @doc_template_provider
        def fa(self, arg):
            """Another docstring %(a)s"""

    # Can be both a user and a provider.
    assert_equal(E().fa.__doc__, "Another docstring 4")
    assert_equal(E().fb.__doc__, "B docstring two")

    @doc_templater(dict(a = 4))
    class F(D):
        @doc_template_provider
        @doc_template_user
        def fa(self, arg):
            """Another docstring %(a)s"""

    # The decorators can be in either order
    assert_equal(F().fa.__doc__, "Another docstring 4")
    assert_equal(F().fb.__doc__, "B docstring two")

    def null_doc_error(err=True):
        @doc_templater(dict(a = 99), doc_error=err)
        class Q(F):
            @doc_template_user
            def fa(self, arg):
                "A docstring too far"
        return Q

    # If docstring to be replaced is not empty, raise error by default
    assert_raises(ValueError, null_doc_error, True)
    assert_equal(null_doc_error(False)().fa.__doc__, "Another docstring 99")

    @doc_templater(dict(a = 5))
    class G(F):
        @doc_template_user
        def fa(self, arg):
            pass

    # Showing the provision from before
    assert_equal(G().fa.__doc__, "Another docstring 5")
    assert_equal(G().fb.__doc__, "B docstring two")

    @doc_templater(dict(a=6, b='three'))
    class H(F):
        @doc_template_provider
        def fa(self, arg):
            """Yet another docstring %(a)s"""

    # Providing without using, again
    assert_equal(H().fa.__doc__, None)

    @doc_templater()
    class I(H):
        @doc_template_user
        def fa(self, arg):
            pass

        @doc_template_user
        def fb(self, arg):
            pass

    # Providing without using, again
    assert_equal(I().fa.__doc__, "Yet another docstring 6")
    assert_equal(I().fb.__doc__, "B docstring three")



"""
A module for commonly used losses that are not in `regreg.smooth.glm`. Only
current example is Huberized SVM.

"""

import numpy as np

from . import smooth_atom
from ..affine import astransform
from ..identity_quadratic import identity_quadratic
from ..atoms.seminorms import positive_part

class huberized_svm(smooth_atom):

    objective_template = r"""\ell^{\text{Huber SVM}}\left(%(var)s\right)"""

    def __init__(self, 
                 X, 
                 labels,
                 smoothing_parameter,
                 coef=1., 
                 offset=None,
                 quadratic=None,
                 initial=None):

        shape = astransform(X).input_shape
        smooth_atom.__init__(self,
                             shape,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial,
                             coef=coef)
        
        if len(np.unique(labels)) > 2:
            raise ValueError('labels should be binary')
        # force labels to be \pm 1
        Y = 2 * (labels == np.max(labels)) - 1
        atom = positive_part.affine(-Y[:, None] * X, np.ones_like(Y), lagrange=1.)
        Q = identity_quadratic(smoothing_parameter, 0, 0, 0)
        self.smoothed_atom = atom.smoothed(Q)
                                         
    def smooth_objective(self, beta, mode='both', check_feasibility=False):
        """

        Parameters
        ----------

        beta : ndarray
            The current parameter values.

        mode : str
            One of ['func', 'grad', 'both']. 

        check_feasibility : bool
            If True, return `np.inf` when
            point is not feasible, i.e. when `beta` is not
            in the domain.

        Returns
        -------

        If `mode` is 'func' returns just the objective value 
        at `beta`, else if `mode` is 'grad' returns the gradient
        else returns both.
        """
        
        return self.smoothed_atom.smooth_objective(beta,
                                                   mode=mode,
                                                   check_feasibility=check_feasibility)

